@@ -993,7 +993,7 @@ pango_fc_font_create_hb_font (PangoFont *font)
 
   if (key)
     {
-      FcPattern *pattern = pango_fc_font_key_get_pattern (key);
+      const FcPattern *pattern = pango_fc_font_key_get_pattern (key);
       const char *variations;
       int index;
       unsigned int n_axes;
@@ -1035,3 +1035,51 @@ pango_fc_font_create_hb_font (PangoFont *font)
 done:
   return hb_font;
 }
+
+/**
+ * pango_fc_font_get_languages:
+ * @font: a #PangoFcFont
+ *
+ * Returns the languages that are supported by @font.
+ *
+ * This corresponds to the FC_LANG member of the FcPattern.
+ *
+ * The returned array is only valid as long as the font
+ * and its fontmap are valid.
+ *
+ * Returns: (transfer none) (nullable): a %NULL-terminated
+ *    array of PangoLanguage*
+ *
+ * Since: 1.48
+ */
+PangoLanguage **
+pango_fc_font_get_languages (PangoFcFont *font)
+{
+  PangoFcFontMap *fontmap;
+  PangoLanguage **languages;
+
+  fontmap = g_weak_ref_get ((GWeakRef *) &font->fontmap);
+  if (!fontmap)
+    return NULL;
+
+  languages  = _pango_fc_font_map_get_languages (fontmap, font);
+  g_object_unref (fontmap);
+
+  return languages;
+}
+
+/**
+ * pango_fc_font_get_pattern:
+ * @font: a #PangoFcFont
+ *
+ * Returns the FcPattern that @font is based on.
+ *
+ * Returns: the fontconfig pattern for this font
+ *
+ * Since: 1.48
+ */
+FcPattern *
+pango_fc_font_get_pattern (PangoFcFont *font)
+{
+  return font->font_pattern;
+}
