@@ -166,8 +166,9 @@ struct _PangoFcFontFaceData
   int id;            /* needed to handle TTC files with multiple faces */
 
   /* Data */
-  FcPattern *pattern;	/* Referenced pattern that owns filename */
+  FcPattern *pattern;  /* Referenced pattern that owns filename */
   PangoCoverage *coverage;
+  PangoLanguage **languages;
 
   hb_face_t *hb_face;
 };
@@ -307,6 +308,8 @@ pango_fc_font_face_data_free (PangoFcFontFaceData *data)
   if (data->coverage)
     pango_coverage_unref (data->coverage);
 
+  g_free (data->languages);
+
   hb_face_destroy (data->hb_face);
 
   g_slice_free (PangoFcFontFaceData, data);
@@ -1337,6 +1340,9 @@ pango_fc_font_map_finalize (GObject *object)
 
   pango_fc_font_map_shutdown (fcfontmap);
 
+  if (fcfontmap->substitute_destroy)
+    fcfontmap->substitute_destroy (fcfontmap->substitute_data);
+
   G_OBJECT_CLASS (pango_fc_font_map_parent_class)->finalize (object);
 }
 
@@ -1798,6 +1804,28 @@ pango_fc_default_substitute (PangoFcFontMap    *fontmap,
     PANGO_FC_FONT_MAP_GET_CLASS (fontmap)->default_substitute (fontmap, pattern);
 }
 
+void
+pango_fc_font_map_set_default_substitute (PangoFcFontMap        *fontmap,
+					  PangoFcSubstituteFunc func,
+					  gpointer              data,
+					  GDestroyNotify        notify)
+{
+  if (fontmap->substitute_destroy)
+    fontmap->substitute_destroy (fontmap->substitute_data);
+
+  fontmap->substitute_func = func;
+  fontmap->substitute_data = data;
+  fontmap->substitute_destroy = notify;
+
+  pango_fc_font_map_substitute_changed (fontmap);
+}
+
+void
+pango_fc_font_map_substitute_changed (PangoFcFontMap *fontmap) {
+  pango_fc_font_map_cache_clear(fontmap);
+  pango_font_map_changed(PANGO_FONT_MAP (fontmap));
+}
+
 static double
 pango_fc_font_map_get_resolution (PangoFcFontMap *fcfontmap,
 				  PangoContext   *context)
@@ -1920,14 +1948,12 @@ pango_fc_fontset_cache (PangoFcFontset *fontset,
     {
       /* Add to cache initially
        */
-#if 1
       if (cache->length == FONTSET_CACHE_SIZE)
 	{
 	  PangoFcFontset *tmp_fontset = g_queue_pop_tail (cache);
 	  tmp_fontset->cache_link = NULL;
 	  g_hash_table_remove (priv->fontset_hash, tmp_fontset->key);
 	}
-#endif
 
       fontset->cache_link = g_list_prepend (NULL, fontset);
     }
@@ -2247,6 +2273,57 @@ _pango_fc_font_map_fc_to_coverage (FcCharSet *charset)
   return (PangoCoverage *)coverage;
 }
 
+static PangoLanguage **
+_pango_fc_font_map_fc_to_languages (FcLangSet *langset)
+{
+  FcStrSet *strset;
+  FcStrList *list;
+  FcChar8 *s;
+  GArray *langs;
+
+  langs = g_array_new (TRUE, FALSE, sizeof (PangoLanguage *));
+
+  strset = FcLangSetGetLangs (langset);
+  list = FcStrListCreate (strset);
+
+  FcStrListFirst (list);
+  while ((s = FcStrListNext (list)))
+    {
+      PangoLanguage *l = pango_language_from_string ((const char *)s);
+      g_array_append_val (langs, l);
+    }
+
+  FcStrListDone (list);
+  FcStrSetDestroy (strset);
+
+  return (PangoLanguage **) g_array_free (langs, FALSE);
+}
+
+PangoLanguage **
+_pango_fc_font_map_get_languages (PangoFcFontMap *fcfontmap,
+                                  PangoFcFont    *fcfont)
+{
+  PangoFcFontFaceData *data;
+  FcLangSet *langset;
+
+  data = pango_fc_font_map_get_font_face_data (fcfontmap, fcfont->font_pattern);
+  if (G_UNLIKELY (!data))
+    return NULL;
+
+  if (G_UNLIKELY (data->languages == NULL))
+    {
+      /*
+       * Pull the languages out of the pattern, this
+       * doesn't require loading the font
+       */
+      if (FcPatternGetLangSet (fcfont->font_pattern, FC_LANG, 0, &langset) != FcResultMatch)
+        return NULL;
+
+      data->languages = _pango_fc_font_map_fc_to_languages (langset);
+    }
+
+  return data->languages;
+}
 /**
  * pango_fc_font_map_create_context:
  * @fcfontmap: a #PangoFcFontMap
@@ -2728,6 +2805,32 @@ create_face (PangoFcFamily *fcfamily,
   return face;
 }
 
+static int
+compare_face (const void *p1, const void *p2)
+{
+  const PangoFcFace *f1 = *(const void **)p1;
+  const PangoFcFace *f2 = *(const void **)p2;
+  int w1, w2;
+  int s1, s2;
+
+  if (FcPatternGetInteger (f1->pattern, FC_WEIGHT, 0, &w1) != FcResultMatch)
+    w1 = FC_WEIGHT_MEDIUM;
+
+  if (FcPatternGetInteger (f1->pattern, FC_SLANT, 0, &s1) != FcResultMatch)
+    s1 = FC_SLANT_ROMAN;
+
+  if (FcPatternGetInteger (f2->pattern, FC_WEIGHT, 0, &w2) != FcResultMatch)
+    w2 = FC_WEIGHT_MEDIUM;
+
+  if (FcPatternGetInteger (f2->pattern, FC_SLANT, 0, &s2) != FcResultMatch)
+    s2 = FC_SLANT_ROMAN;
+
+  if (s1 != s2)
+    return s1 - s2; /* roman < italic < oblique */
+
+  return w1 - w2; /* from light to heavy */
+}
+
 static void
 ensure_faces (PangoFcFamily *fcfamily)
 {
@@ -2837,6 +2940,8 @@ ensure_faces (PangoFcFamily *fcfamily)
 
 	  faces = g_renew (PangoFcFace *, faces, num);
 
+          qsort (faces, num, sizeof (PangoFcFace *), compare_face);
+
 	  fcfamily->n_faces = num;
 	  fcfamily->faces = faces;
 	}
